"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.nodeExternals = void 0;
const build_utils_1 = require("../build-utils");
const cwd_1 = require("../cwd");
const static_1 = require("../static");
const external_utils_1 = require("./external-utils");
const scopedModuleRegex = new RegExp('@[a-zA-Z0-9][\\w-.]+/[a-zA-Z0-9][\\w-.]+([a-zA-Z0-9./]+)?', 'g');
function getModuleName(request, includeAbsolutePaths) {
    let req = request;
    const delimiter = '/';
    if (includeAbsolutePaths) {
        req = req.replace(/^.*?\/node_modules\//, '');
    }
    // check if scoped module
    if (scopedModuleRegex.test(req)) {
        // reset regexp
        scopedModuleRegex.lastIndex = 0;
        return req.split(delimiter, 2).join(delimiter);
    }
    return req.split(delimiter)[0];
}
function wrap(whitelist) {
    const allDependencies = {};
    whitelist.forEach((item) => {
        if (typeof item === 'string') {
            const entryFile = (0, cwd_1.requireWithPreserveLinks)(item);
            if (entryFile) {
                (0, build_utils_1.getDependencies)(entryFile, allDependencies);
            }
        }
    });
    return whitelist.concat(Object.keys(allDependencies).map((item) => new RegExp(item)));
}
function nodeExternals(options) {
    options = options || {};
    const whitelist = wrap([].concat(options.whitelist || []));
    const binaryDirs = [].concat(options.binaryDirs || ['.bin']);
    const importType = options.importType || 'commonjs';
    const modulesDir = options.modulesDir || 'node_modules';
    const modulesFromFile = !!options.modulesFromFile;
    const includeAbsolutePaths = !!options.includeAbsolutePaths;
    function isNotBinary(x) {
        return !(0, external_utils_1.contains)(binaryDirs, x);
    }
    // create the node modules list
    let nodeModules = [];
    if (modulesFromFile) {
        nodeModules = (0, external_utils_1.readFromPackageJson)(options.modulesFromFile);
    }
    else {
        if (Array.isArray(modulesDir)) {
            modulesDir.map((str) => {
                nodeModules = nodeModules.concat((0, external_utils_1.readDir)(str).filter(isNotBinary));
            });
        }
        else {
            nodeModules = (0, external_utils_1.readDir)(modulesDir).filter(isNotBinary);
        }
    }
    return function (_context, request, callback) {
        if (!options.isServer) {
            if (request.startsWith('@external:')) {
                // set external in client
                return callback(null, importType + ' ' + request.slice(10));
            }
            else {
                return callback();
            }
        }
        if (request.startsWith('@external:')) {
            return callback(null, importType + ' ' + request.slice(10));
        }
        const moduleName = getModuleName(request, includeAbsolutePaths);
        if (static_1.nameSpaceBuiltinModules.includes(moduleName)) {
            // external node nartive module
            return callback(null, importType + ' ' + request);
        }
        if ((0, external_utils_1.contains)(nodeModules, moduleName) || static_1.defaultExternal.includes(moduleName)) {
            if ((0, external_utils_1.containsPattern)(whitelist, request)) {
                // 白名单中的一定需要被处理
                return callback();
            }
            // 否则 被 external
            // mark this module as external
            // https://webpack.js.org/configuration/externals/
            return callback(null, importType + ' ' + request);
        }
        callback();
    };
}
exports.nodeExternals = nodeExternals;
//# sourceMappingURL=externals.js.map